import os
import uuid

import orjson
from semver import VersionInfo

from vdb.lib import (
    config,
    CPE_FULL_REGEX,
    KNOWN_PKG_TYPES,
    PKG_TYPES_MAP,
    Vulnerability,
    VulnerabilityDetail,
    VulnerabilitySource,
)
from vdb.lib import db6 as db_lib
from vdb.lib.cna import ASSIGNER_UUID_MAP
from vdb.lib.cve_model import (
    CVE,
    CVE1,
    Affected,
    CnaPublishedContainer,
    Containers,
    Cpe,
    CveId,
    CveMetadataPublished,
    DataType,
    DataVersion,
    Description,
    Description1,
    Language,
    Metrics,
    Metrics1,
    Metrics2,
    OrgId,
    ProblemType,
    ProblemTypes,
    Product,
    ProviderMetadata,
    Reference,
    References,
    State,
    Status,
    UuidType,
    Version,
    Versions,
)
from vdb.lib.cve_model.common import CiaType
from vdb.lib.cve_model.cvss_v3 import Field0, Field1
from vdb.lib.utils import calculate_hash, to_purl_vers

# Our DB creation process could result in duplicates. By tracking these keys we reduce them
source_completed_keys = {}
index_completed_keys = {}


def to_cve_metadata(avuln: Vulnerability):
    # Re-use the assigner org id
    if avuln.assigner:
        if ASSIGNER_UUID_MAP.get(avuln.assigner):
            assigner_org_id = ASSIGNER_UUID_MAP[avuln.assigner]
        else:
            # We have encountered a new assigner
            assigner_org_id = OrgId(UuidType(str(uuid.uuid4())))
            ASSIGNER_UUID_MAP[avuln.assigner] = assigner_org_id
    else:
        assigner_org_id = OrgId(UuidType(str(uuid.uuid4())))
    metadata = CveMetadataPublished(
        cveId=CveId(avuln.id),
        state=State.PUBLISHED,
        assignerOrgId=assigner_org_id,
        assignerShortName=avuln.assigner,
    )
    metadata.datePublished = avuln.source_orig_time
    metadata.dateUpdated = avuln.source_update_time
    return metadata


def all_semver_compatible(adetail: VulnerabilityDetail) -> bool:
    if adetail.mii and adetail.mii != "*" and not VersionInfo.is_valid(adetail.mii):
        return False
    if adetail.mae and adetail.mae != "*" and not VersionInfo.is_valid(adetail.mae):
        return False
    if adetail.mie and adetail.mie != "*" and not VersionInfo.is_valid(adetail.mie):
        return False
    if adetail.mai and adetail.mai != "*" and not VersionInfo.is_valid(adetail.mai):
        return False
    return True


def to_product_versions(vendor, adetail: VulnerabilityDetail) -> list[Versions]:
    versions = []
    lt_captured = False
    # Try to detect the version_type
    version_type = "custom"
    if vendor in KNOWN_PKG_TYPES:
        version_type = vendor
    elif all_semver_compatible(adetail):
        version_type = "semver"
    # The goal is to create a single entry with either version and lessThan
    #   or version and lessThanOrEqual
    # If this is not possible then we create a separate version objects
    if adetail.mii and adetail.mii not in ("*", "-"):
        if adetail.mai and adetail.mai not in ("*", "-"):
            versions.append(
                Versions(
                    version=Version(adetail.mii),
                    lessThanOrEqual=Version(adetail.mai),
                    versionType=version_type,
                    status=Status.affected,
                )
            )
            lt_captured = True
        elif adetail.mae:
            versions.append(
                Versions(
                    version=Version(adetail.mii),
                    lessThan=Version(adetail.mae),
                    versionType=version_type,
                    status=Status.affected,
                )
            )
            lt_captured = True
        else:
            versions.append(
                Versions(
                    version=Version(adetail.mii),
                    lessThanOrEqual=Version(adetail.mai),
                    versionType=version_type,
                    status=Status.affected,
                )
            )
    if adetail.mie and adetail.mie not in ("*", "-"):
        versions.append(
            Versions(
                version=Version(adetail.mie),
                versionType=version_type,
                status=Status.unaffected,
            )
        )
    if not lt_captured:
        if adetail.mai and adetail.mai not in ("*", "-"):
            versions.append(
                Versions(
                    version=Version("0"),
                    lessThanOrEqual=Version(adetail.mai),
                    versionType=version_type,
                    status=Status.affected,
                )
            )
        if adetail.mae and adetail.mae not in ("*", "-"):
            versions.append(
                Versions(
                    version=Version("0"),
                    lessThan=Version(adetail.mae),
                    versionType=version_type,
                    status=Status.affected,
                )
            )
    return versions


def to_cve_affected(avuln: Vulnerability) -> Affected | None:
    products = []
    adetail: VulnerabilityDetail
    for adetail in avuln.details:
        cpe_uri = adetail.cpe_uri
        parts = CPE_FULL_REGEX.match(cpe_uri)
        if parts:
            versions = to_product_versions(parts.group("vendor"), adetail)
            if versions:
                # Similar to purl type
                vendor = parts.group("vendor")
                # Similar to purl namespace
                product = parts.group("package").removesuffix("\\").removesuffix("!")
                # Similar to purl name
                package_name = parts.group("package")
                if "/" in product:
                    tmp_a = product.split("/")
                    # ubuntu/upstream/virtualbox should become
                    # product=ubuntu and package_name=upstream/virtualbox
                    if vendor in config.OS_PKG_TYPES or config.VENDOR_TO_VERS_SCHEME.get(vendor):
                        product = tmp_a[0]
                        package_name = "/".join(tmp_a[1:])
                    elif len(tmp_a) != 2:
                        if len(tmp_a) > 2 and vendor in ("generic", "swift"):
                            product = os.path.dirname(product)
                            package_name = os.path.basename(package_name)
                            # If we get an empty package_name then fallback to using the full string as package_name
                            if not package_name:
                                product = None
                                package_name = parts.group("package")
                        else:
                            product = None
                    elif vendor not in ("golang",):
                        product = tmp_a[0]
                        package_name = tmp_a[1]
                # Product and package name are the same.
                # For some ecosystems, we can remove the product (namespace) to rely only on name
                # For others, we can make the product the same as vendor
                if product == package_name:
                    if vendor in ("npm", "pypi", "gem", "swift"):
                        product = None
                    elif vendor not in (
                        "maven",
                        "composer",
                        "generic",
                        "github",
                        "gitlab",
                    ):
                        product = vendor
                # See if we can substitute vers scheme
                if config.VENDOR_TO_VERS_SCHEME.get(vendor):
                    vendor = config.VENDOR_TO_VERS_SCHEME.get(vendor)
                # This prevents cargo:cargo or nuget:nuget
                # or openssl:openssl:openssl
                # but retain such values for github and gitlab
                if (
                    product == vendor
                    and vendor not in ("github", "gitlab")
                    and (package_name == product or vendor in KNOWN_PKG_TYPES)
                ):
                    product = None
                # Deal with NVD mess such as npmjs or crates
                if vendor not in KNOWN_PKG_TYPES:
                    for k, v in PKG_TYPES_MAP.items():
                        if vendor.lower() in v:
                            vendor = k
                            break
                p = Product(
                    vendor=vendor,
                    product=product,
                    packageName=package_name,
                    cpes=[Cpe(cpe_uri)],
                    defaultStatus=Status.unknown,
                    versions=versions,
                )
                products.append(p)
    return Affected(products) if products else None


def to_complexity_type(complexity: str) -> str:
    if complexity and complexity.lower() == "low":
        return "LOW"
    return "HIGH"


def severity_to_impact(severity: str) -> CiaType:
    """Method to convert severity string to a valid impact enum string"""
    if not severity:
        return CiaType.NONE
    if severity.lower() in ("critical", "high", "medium", "moderate"):
        return CiaType.HIGH
    return CiaType.LOW


def to_metrics(avuln: Vulnerability) -> Metrics:
    metrics_list = []
    if avuln.cvss_v3.vector_string and avuln.cvss_v3.vector_string.startswith(
        "CVSS:3.1"
    ):
        metrics_list.append(
            Metrics1(
                cvssV3_1=Field1(
                    version="3.1",
                    baseScore=avuln.cvss_v3.base_score,
                    baseSeverity=config.THREAT_TO_SEVERITY.get(avuln.severity.lower()),
                    vectorString=avuln.cvss_v3.vector_string,
                    attackVector=avuln.cvss_v3.attack_vector,
                    attackComplexity=to_complexity_type(
                        avuln.cvss_v3.attack_complexity
                    ),
                    privilegesRequired=avuln.cvss_v3.privileges_required,
                    userInteraction=avuln.cvss_v3.user_interaction,
                    scope=avuln.cvss_v3.scope,
                    confidentialityImpact=severity_to_impact(
                        avuln.cvss_v3.confidentiality_impact
                    ),
                    integrityImpact=severity_to_impact(avuln.cvss_v3.integrity_impact),
                    availabilityImpact=severity_to_impact(
                        avuln.cvss_v3.availability_impact
                    ),
                )
            )
        )
    elif avuln.cvss_v3.vector_string and avuln.cvss_v3.vector_string.startswith(
        "CVSS:3.0"
    ):
        metrics_list.append(
            Metrics2(
                cvssV3_0=Field0(
                    version="3.0",
                    baseScore=avuln.cvss_v3.base_score,
                    baseSeverity=config.THREAT_TO_SEVERITY.get(avuln.severity.lower()),
                    vectorString=avuln.cvss_v3.vector_string,
                    attackVector=avuln.cvss_v3.attack_vector,
                    attackComplexity=avuln.cvss_v3.attack_complexity,
                    privilegesRequired=avuln.cvss_v3.privileges_required,
                    userInteraction=avuln.cvss_v3.user_interaction,
                    scope=avuln.cvss_v3.scope,
                    confidentialityImpact=severity_to_impact(
                        avuln.cvss_v3.confidentiality_impact
                    ),
                    integrityImpact=severity_to_impact(avuln.cvss_v3.integrity_impact),
                    availabilityImpact=severity_to_impact(
                        avuln.cvss_v3.availability_impact
                    ),
                )
            )
        )

    return Metrics(metrics_list)


def to_references(avuln: Vulnerability) -> list[Reference] | None:
    ref_list = [
        Reference(url=url) for url in avuln.related_urls if url.startswith("http")
    ]
    return References(ref_list) if ref_list else None


def to_cve_containers(avuln: Vulnerability) -> CnaPublishedContainer | None:
    provier_meta = ProviderMetadata(
        orgId=ASSIGNER_UUID_MAP.get(avuln.assigner, OrgId(UuidType(str(uuid.uuid4()))))
    )
    provier_meta.dateUpdated = avuln.source_update_time
    affected = to_cve_affected(avuln)
    if not affected:
        return None
    cont = CnaPublishedContainer(
        providerMetadata=provier_meta,
        descriptions=[Description(lang=Language("en"), value=avuln.description)],
        affected=affected,
        metrics=to_metrics(avuln),
    )
    references = to_references(avuln)
    if references:
        cont.references = references
    cont.dateAssigned = avuln.source_orig_time
    # CWE
    if avuln.problem_type and "noinfo" not in avuln.problem_type:
        problem_types = []
        for acwe in avuln.problem_type.split(","):
            # Check if this starts with CWE
            if acwe.startswith("CWE"):
                problem_types.append(
                    ProblemType(
                        descriptions=[
                            Description1(
                                lang=Language("en"),
                                description=acwe,
                                cweId=acwe,
                                type="CWE",
                            )
                        ]
                    )
                )
        if problem_types:
            cont.problemTypes = ProblemTypes(problem_types)
    return cont


class CVESource(VulnerabilitySource):
    """
    Generic CVE source that uses the CVE 5.0 models
    """

    db_conn = None
    index_conn = None

    def __init__(self):
        self.db_conn, self.index_conn = db_lib.get(
            db_file=config.VDB_BIN_FILE, index_file=config.VDB_BIN_INDEX
        )

    @classmethod
    def download_all(cls):
        db_lib.clear_all()

    @classmethod
    def download_recent(cls):
        pass

    @classmethod
    def bulk_search(cls, app_info, pkg_list):
        pass

    @classmethod
    def refresh(cls):
        pass

    def convert(self, cve_data: dict) -> list[Vulnerability]:
        pass

    def convert5(self, data: list[Vulnerability]) -> list[CVE]:
        cves = []
        for avuln in data:
            containers = Containers(cna=to_cve_containers(avuln))
            if containers:
                cve_obj = CVE1(
                    dataType=DataType.CVE_RECORD,
                    dataVersion=DataVersion.field_5_0,
                    cveMetadata=to_cve_metadata(avuln),
                    containers=containers,
                )
                cves.append(cve_obj)
        return cves

    def store(self, data: list[Vulnerability]):
        """Store data in the database"""
        cve5_list = self.convert5(data)
        self.store5(cve5_list)

    def store5(self, data: list[CVE]):
        """Store the CVE and index data into the SQLite database"""
        with self.db_conn as dbc:
            with self.index_conn as indexc:
                for d in data:
                    cve_id = d.cveMetadata.cveId
                    cve_id = cve_id.model_dump(mode="python")
                    source_data = d.model_dump(
                        mode="json",
                        exclude_defaults=True,
                        exclude_unset=True,
                        exclude_none=True,
                    )
                    source_data_str = orjson.dumps(source_data).decode(
                        "utf-8", "ignore"
                    )
                    source_hash = calculate_hash(source_data_str)
                    if d.containers.cna and d.containers.cna.affected:
                        for affected in d.containers.cna.affected.root:
                            vers = to_purl_vers(affected.vendor, affected.versions)
                            purl_type = (
                                affected.vendor
                                if affected.vendor in KNOWN_PKG_TYPES
                                else "generic"
                            )
                            purl_prefix = f"""pkg:{purl_type}/"""
                            if affected.product:
                                purl_prefix = f"{purl_prefix}{affected.product}/"
                            purl_prefix = f"{purl_prefix}{affected.packageName}"
                            pkg_key = f"{cve_id}|{affected.vendor}|{affected.product}|{affected.packageName}|{source_hash}"
                            index_pkg_key = f"{cve_id}|{affected.vendor}|{affected.product}|{affected.packageName}|{vers}"
                            # Filter obvious duplicates
                            if not source_completed_keys.get(pkg_key):
                                dbc.execute(
                                    "INSERT INTO cve_data values(?, ?, ?, ?, jsonb(?), ?, ?, ?);",
                                    (
                                        cve_id,
                                        affected.vendor,
                                        affected.product,
                                        affected.packageName,
                                        source_data_str,
                                        None,
                                        source_hash,
                                        purl_prefix,
                                    ),
                                )
                                source_completed_keys[pkg_key] = True
                            if not index_completed_keys.get(index_pkg_key):
                                indexc.execute(
                                    "INSERT INTO cve_index values(?, ?, ?, ?, ?, ?);",
                                    (
                                        cve_id,
                                        affected.vendor,
                                        affected.product,
                                        affected.packageName,
                                        vers,
                                        purl_prefix,
                                    ),
                                )
                                index_completed_keys[index_pkg_key] = True



from typing import Any, Generator

import orjson

from vdb.lib import db6, utils
from vdb.lib.cve_model import CVE, CVE1


def _filter_hits(raw_hits: list, compare_ver: str) -> list:
    filtered_list = []
    for ahit in raw_hits:
        cve_id = ahit[0]
        vers = ahit[4]
        if utils.vers_compare(compare_ver, vers):
            filtered_list.append(
                {
                    "cve_id": cve_id,
                    "type": ahit[1],
                    "namespace": ahit[2],
                    "name": ahit[3],
                    "vers": vers,
                    "purl_prefix": ahit[-1],
                }
            )
    return filtered_list


def get_cve_data(
    db_conn, index_hits: list[dict, Any], search_str: str
) -> Generator | list[dict[str, str | CVE | None]]:
    """Get CVE data for the index results

    Args:
        db_conn: DB Connection or None to create a new one
        index_hits: Hits from one of the search methods
        search_str: Original search string used

    Returns:
        generator: generator for CVE data with original source data as a pydantic model
    """
    if not db_conn:
        db_conn, _ = db6.get(read_only=True)
    for ahit in index_hits:
        results = exec_query(
            db_conn,
            "SELECT DISTINCT cve_id, type, namespace, name, source_data_hash, json(source_data), json(override_data), purl_prefix FROM cve_data WHERE cve_id = ? AND purl_prefix = ? GROUP BY purl_prefix ORDER BY cve_id DESC;",
            (ahit["cve_id"], ahit["purl_prefix"]),
        )
        for res in results:
            yield {
                "cve_id": res[0],
                "type": res[1],
                "namespace": res[2],
                "name": res[3],
                "matching_vers": ahit["vers"],
                "matched_by": search_str,
                "source_data_hash": res[4],
                "source_data": (
                    CVE(root=CVE1.model_validate(orjson.loads(res[5]), strict=False))
                    if res[5]
                    else None
                ),
                "override_data": (orjson.loads(res[6]) if res[6] else None),
                "purl_prefix": res[7],
            }


def search_by_any(any_str: str, with_data: bool = False) -> list | None:
    """Convenient method to search by a string"""
    if any_str.startswith("pkg:"):
        return search_by_purl_like(any_str, with_data)
    if (
        any_str.startswith("CVE-")
        or any_str.startswith("GHSA-")
        or any_str.startswith("MAL-")
    ):
        return search_by_cve(any_str, with_data)
    if any_str.startswith("http"):
        return search_by_url(any_str, with_data)
    return search_by_cpe_like(any_str, with_data)


def search_by_cpe_like(cpe: str, with_data=False) -> list | None:
    """Search by CPE or colon-separate strings"""
    db_conn, index_conn = db6.get(read_only=True)
    if cpe.startswith("cpe:"):
        vendor, package, version, _ = utils.parse_cpe(cpe)
    elif cpe.count(":") == 2:
        vendor, package, version = cpe.split(":")
    else:
        return None
    # check for vendor name in both namespace and type
    raw_hits = exec_query(
        index_conn,
        "SELECT DISTINCT cve_id, type, namespace, name, vers, purl_prefix FROM cve_index where (namespace = ? OR type = ?) AND name = ?;",
        (vendor, vendor, package),
    )
    filtered_list = _filter_hits(raw_hits, version)
    if with_data:
        return get_cve_data(db_conn, filtered_list, cpe)
    return filtered_list


def search_by_purl_like(purl: str, with_data=False) -> list | None:
    """Search by purl like string"""
    db_conn, index_conn = db6.get(read_only=True)
    purl_obj = utils.parse_purl(purl)
    if purl_obj:
        ptype = purl_obj.get("type")
        namespace = purl_obj.get("namespace")
        name = purl_obj.get("name")
        version = purl_obj.get("version", "*")
        purl_prefix = f"pkg:{ptype}/"
        if namespace:
            purl_prefix = f"{purl_prefix}{namespace}/"
        # Handle distro names for linux os purls by prefixing distro name to name
        if purl_obj["qualifiers"] and purl_obj["qualifiers"].get("distro_name"):
            distro_name = purl_obj["qualifiers"].get("distro_name")
            name = f"{distro_name}/{name}"
        purl_prefix = f"{purl_prefix}{name}"
        args = (purl_prefix,)
        raw_hits = exec_query(
            index_conn,
            "SELECT DISTINCT cve_id, type, namespace, name, vers, purl_prefix FROM cve_index where purl_prefix = ?;",
            args,
        )
        filtered_list = _filter_hits(raw_hits, version)
        if with_data:
            return get_cve_data(db_conn, filtered_list, purl)
        return filtered_list
    return None


def search_by_cve(cve_id: str, with_data=False, with_limit=None) -> list | None:
    """Search by CVE"""
    db_conn, index_conn = db6.get(read_only=True)
    filter_part = "cve_id LIKE ?" if "%" in cve_id else "cve_id = ?"
    filter_part = f"{filter_part} ORDER BY cve_id DESC"
    args = [cve_id]
    if with_limit:
        filter_part = f"{filter_part} LIMIT ?"
        args.append(with_limit)
    raw_hits = exec_query(
        index_conn,
        f"SELECT DISTINCT cve_id, type, namespace, name, vers, purl_prefix FROM cve_index where {filter_part}",
        args,
    )
    filtered_list = _filter_hits(raw_hits, "*")
    if with_data:
        return get_cve_data(db_conn, filtered_list, cve_id)
    return filtered_list


def search_by_url(url: str, with_data=False) -> list | None:
    """Search by URL"""
    purl_obj = utils.url_to_purl(url)
    if not purl_obj:
        return None
    name = purl_obj["name"]
    purl_str = (
        f"pkg:{purl_obj['type']}/{purl_obj['namespace']}/{name}"
        if purl_obj["namespace"]
        else f"pkg:{purl_obj['type']}/{name}"
    )
    if purl_obj["version"]:
        purl_str = f"{purl_str}@{purl_obj['version']}"
    return search_by_purl_like(purl_str, with_data)


def search_by_cdx_bom(bom_file: str, with_data=False) -> Generator:
    """Search by CycloneDX BOM file"""
    with open(bom_file, encoding="utf-8", mode="r") as fp:
        cdx_obj = orjson.loads(fp.read())
        for component in cdx_obj.get("components"):
            if component.get("purl"):
                yield search_by_purl_like(component.get("purl"), with_data)
            if component.get("cpe"):
                yield search_by_cpe_like(component.get("cpe"), with_data)


def latest_malware(with_limit=20, with_data=False) -> Generator:
    yield search_by_cve("MAL-%", with_data=with_data, with_limit=with_limit)


def exec_query(conn, query: str, args: tuple[str, ...]) -> list:
    res = conn.execute(query, args)
    return res.fetchall()



import json
import os

import pytest

from vdb.lib import VulnerabilityLocation, db6, search
from vdb.lib.aqua import AquaSource
from vdb.lib.cve import CVESource
from vdb.lib.gha import GitHubSource
from vdb.lib.nvd import NvdSource
from vdb.lib.osv import OSVSource


@pytest.fixture
def test_cve_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "cve_data.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_cve_wconfig_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "cve_wconfig_data.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_nvd_api_json1():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2024-0057.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_nvd_api_json2():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2024-21312.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_nvd_api_json3():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2024-23771.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_nvd_api_json4():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2015-3192.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_nvd_api_git_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2023-52426.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_rust_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "osv_rust_data.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_mvn_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "osv_mvn_data.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_mixed_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "osv_mixed_data.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_go_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "GO-2022-0646.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_pypi_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "osv-pypi.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_pypi2_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "osv-pypi2.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_swift_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "osv_swift_data.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_swift2_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "osv_swift_data2.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_mevents_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "osv_multi_events.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_osv_git_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "osv-git.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_alsa_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "ALSA-2022-8580.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_alas_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "ALAS2022-2022-207.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_rlsa_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "RLSA-2022-7730.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_ubuntu_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-45406.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_ubuntu2_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-3715.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_ubuntu1_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-6083.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_ubuntu3_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-3219.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_ubuntu4_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-32081.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_ubuntu5_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-3821.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_redhat_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-45418.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_redhat2_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-21824.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_arch_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "AVG-999.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_opensuse_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)),
        "data",
        "openSUSE-SU-2022-2801-1.json",
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_suse_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "SUSE-SU-2022-4192-1.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_suse_json1():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "SUSE-SU-2015-0439-1.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_photon_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2021-3618.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_debian_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2019-18625.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_debian2_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2023-21500.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_debian3_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-3567.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_debian4_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "DLA-981-1.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_debian5_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2021-22890.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_debian6_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2022-32091.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_debian7_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2020-35448.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_cg_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "redis.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_wolfi_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "protobuf-c.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


@pytest.fixture
def test_aqua_alpine_unfixed_json():
    test_cve_data = os.path.join(
        os.path.dirname(os.path.realpath(__file__)), "data", "CVE-2024-25450.json"
    )
    with open(test_cve_data, "r") as fp:
        return json.loads(fp.read())


def test_convert(test_cve_json):
    nvdlatest = NvdSource()
    vulnerabilities = nvdlatest.convert(test_cve_json)
    assert len(vulnerabilities) == 384
    for v in vulnerabilities:
        details = v.details
        for detail in details:
            assert detail
            assert detail.severity
            assert detail.package
            assert detail.package_type

    db6.clear_all()
    nvdlatest.store(vulnerabilities)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 496
    assert cve_index_count == 1155
    results_count = len(list(search.search_by_any("CVE-2020-0001")))
    assert results_count == 4
    results_count = len(
        list(search.search_by_any("cpe:2.3:o:google:android:8.1:*:*:*:*:*:*:*"))
    )
    assert results_count == 25

    cvesource = CVESource()
    cve = cvesource.convert5(vulnerabilities)
    assert len(cve) == 384

    db6.clear_all()
    cvesource.store(vulnerabilities)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 0
    assert cve_index_count == 0


def test_convert2(test_cve_wconfig_json):
    nvdlatest = NvdSource()
    vulnerabilities = nvdlatest.convert(test_cve_wconfig_json)
    assert len(vulnerabilities) == 1
    for v in vulnerabilities:
        details = v.details
        for detail in details:
            assert detail
            assert detail.severity
            assert detail.package
            assert detail.package_type
            assert not detail.fixed_location

    db6.clear_all()
    nvdlatest.store(vulnerabilities)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 2
    assert cve_index_count == 4
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 4
    results_count = len(
        list(search.search_by_any("cpe:2.3:o:opensuse:leap:15.1:*:*:*:*:*:*:*"))
    )
    assert results_count == 1

    cvesource = CVESource()
    cve = cvesource.convert5(vulnerabilities)
    assert len(cve) == 1

    db6.clear_all()
    cvesource.store(vulnerabilities)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 0
    assert cve_index_count == 0


def test_nvd_api_convert(
    test_nvd_api_json1,
    test_nvd_api_json2,
    test_nvd_api_json3,
    test_nvd_api_json4,
    test_nvd_api_git_json,
):
    # json1
    nvdlatest = NvdSource()
    vulnerabilities = nvdlatest.convert(test_nvd_api_json1)
    assert len(vulnerabilities) == 1
    for v in vulnerabilities:
        details = v.details
        for detail in details:
            assert detail
            assert detail.severity
            assert detail.package
            assert detail.package_type
            assert not detail.fixed_location

    db6.clear_all()
    nvdlatest.store(vulnerabilities)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 4
    assert cve_index_count == 20
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2024-0057")))
    assert results_count == 10
    results_count = len(
        list(search.search_by_any("cpe:2.3:a:microsoft:.net:*:*:*:*:*:*:*:*"))
    )
    assert results_count == 1

    # json2
    vulnerabilities = nvdlatest.convert(test_nvd_api_json2)
    assert len(vulnerabilities) == 1
    cvesource = CVESource()
    cve = cvesource.convert5(vulnerabilities)
    assert len(cve) == 1

    db6.clear_all()
    nvdlatest.store(vulnerabilities)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 7
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2024-21312")))
    assert results_count == 7

    # json3
    vulnerabilities = nvdlatest.convert(test_nvd_api_json3)
    assert len(vulnerabilities) == 0
    cve = cvesource.convert5(vulnerabilities)
    assert len(cve) == 0

    db6.clear_all()
    nvdlatest.store(vulnerabilities)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 0
    assert cve_index_count == 0
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2024-23771")))
    assert results_count == 0

    # json4
    vulnerabilities = nvdlatest.convert(test_nvd_api_json4)
    assert len(vulnerabilities) == 1

    db6.clear_all()
    nvdlatest.store(vulnerabilities)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 2
    assert cve_index_count == 21
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2015-3192")))
    assert results_count == 21
    results_count = len(
        list(
            search.search_by_any(
                "cpe:2.3:a:pivotal_software:spring_framework:3.2.0:*:*:*:*:*:*:*"
            )
        )
    )
    assert results_count == 2

    # git_json
    vulnerabilities = nvdlatest.convert(test_nvd_api_git_json)
    assert len(vulnerabilities) == 1
    assert len(vulnerabilities[0].details) == 2

    db6.clear_all()
    nvdlatest.store(vulnerabilities)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 2
    assert cve_index_count == 2
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2023-52426")))
    assert results_count == 2
    results_count = len(
        list(
            search.search_by_any("cpe:2.3:a:libexpat_project:libexpat:*:*:*:*:*:*:*:*")
        )
    )
    assert results_count == 1


@pytest.mark.skip(reason="This downloads and tests with live data")
def test_nvd_download():
    nvdlatest = NvdSource()
    data = nvdlatest.download_recent()
    assert len(data) > 300


@pytest.mark.skip(reason="This downloads and tests with live data")
def test_download_all():
    nvdlatest = NvdSource()
    nvdlatest.download_all()


@pytest.mark.skip(reason="This downloads and tests with live data")
def test_gha_download():
    ghalatest = GitHubSource()
    data = ghalatest.download_recent()
    assert len(data) > 100


@pytest.mark.skip(reason="This downloads and tests with live data")
def test_gha_download_all():
    ghalatest = GitHubSource()
    data = ghalatest.download_all()
    assert len(data) > 1000


def test_gha_version_ranges():
    source = GitHubSource()
    version_list = source.get_version_range("< 1.10.2")
    assert version_list == ("", "", "", "1.10.2")
    version_list = source.get_version_range("= 0.2.0")
    assert version_list == ("", "0.2.0", "", "")
    version_list = source.get_version_range("<= 1.0.8")
    assert version_list == ("", "1.0.8", "", "")
    version_list = source.get_version_range(">= 4.3.0, < 4.3.5")
    assert version_list == ("4.3.0", "", "", "4.3.5")
    version_list = source.get_version_range(">= 0.5.1")
    assert version_list == ("0.5.1", "", "", "")
    version_list = source.get_version_range("> 2.2.0, < 3.1.8")
    assert version_list == ("", "", "2.2.0", "3.1.8")
    version_list = source.get_version_range("> 2.0.0, <= 2.0.14")
    assert version_list == ("", "2.0.14", "2.0.0", "")
    version_list = source.get_version_range(">= 1.0.0, <= 2.0.14")
    assert version_list == ("1.0.0", "2.0.14", "", "")
    version_list = source.get_version_range("1.1.0")
    assert version_list == ("1.1.0", "", "", "")
    version_list = source.get_version_range("> 11.0, < 24.1.1")
    assert version_list == ("", "", "11.0", "24.1.1")


@pytest.mark.skip(reason="This downloads and tests with live data")
def test_osv_download_all():
    osvlatest = OSVSource()
    data = osvlatest.download_all()
    assert len(data) > 1000


def test_osv_convert(
    test_osv_rust_json,
    test_osv_mvn_json,
    test_osv_mixed_json,
    test_osv_go_json,
    test_osv_pypi_json,
    test_osv_pypi2_json,
    test_osv_swift_json,
    test_osv_swift2_json,
    test_osv_mevents_json,
    test_osv_git_json,
):
    osvlatest = OSVSource()

    # git
    cve_data = osvlatest.convert(test_osv_git_json)
    assert cve_data
    assert len(cve_data) == 5

    db6.clear_all()
    osvlatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 5
    assert cve_index_count == 5
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2019-0647")))
    assert results_count == 2
    results_count = len(list(search.search_by_any("pkg:maven/org.springframework/spring-web")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("pkg:apk/alpine/mariadb?arch=source")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("pkg:apk/alpine/alpine-3.2/mariadb?arch=source")))
    assert results_count == 1

    # multi events
    cve_data = osvlatest.convert(test_osv_mevents_json)
    assert cve_data
    assert len(cve_data) == 4

    db6.clear_all()
    osvlatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 4
    assert cve_index_count == 4
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2019-3192")))
    assert results_count == 1
    results_count = len(list(search.search_by_any("pkg:apk/alpine/mariadb?arch=source")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("pkg:maven/org.springframework/spring-web")))
    assert results_count == 4

    # swift
    cve_data = osvlatest.convert(test_osv_swift_json)
    assert cve_data
    assert len(cve_data) == 2

    db6.clear_all()
    osvlatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 2
    assert cve_index_count == 2

    # swift2
    cve_data = osvlatest.convert(test_osv_swift2_json)
    assert cve_data
    assert len(cve_data) == 2

    db6.clear_all()
    osvlatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 2
    assert cve_index_count == 2
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2024-21631")))
    assert results_count == 2

    # rust
    cve_data = osvlatest.convert(test_osv_rust_json)
    assert cve_data

    db6.clear_all()
    osvlatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1
    results_count = len(list(search.search_by_any("pkg:apk/alpine/mariadb?arch=source")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("pkg:cargo/rusqlite")))
    assert results_count == 1

    # maven
    cve_data = osvlatest.convert(test_osv_mvn_json)
    assert cve_data
    assert len(cve_data) == 3

    db6.clear_all()
    osvlatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 3
    assert cve_index_count == 3

    # mixed
    cve_data = osvlatest.convert(test_osv_mixed_json)
    assert cve_data
    assert len(cve_data) == 3

    db6.clear_all()
    osvlatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 3
    assert cve_index_count == 3
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2021-23440")))
    assert results_count == 2

    # go
    cve_data = osvlatest.convert(test_osv_go_json)
    assert cve_data
    assert len(cve_data) == 1

    db6.clear_all()
    osvlatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2020-8911")))
    assert results_count == 1

    # pypi
    cve_data = osvlatest.convert(test_osv_pypi_json)
    assert cve_data
    assert len(cve_data) == 1

    db6.clear_all()
    osvlatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2021-25951")))
    assert results_count == 1

    # pypi2
    cve_data = osvlatest.convert(test_osv_pypi2_json)
    assert not cve_data


def test_aqua_convert(
    test_aqua_alsa_json,
    test_aqua_alas_json,
    test_aqua_rlsa_json,
    test_aqua_ubuntu_json,
    test_aqua_ubuntu1_json,
    test_aqua_ubuntu2_json,
    test_aqua_ubuntu3_json,
    test_aqua_redhat_json,
    test_aqua_redhat2_json,
    test_aqua_arch_json,
    test_aqua_opensuse_json,
    test_aqua_suse_json,
    test_aqua_suse_json1,
    test_aqua_photon_json,
    test_aqua_debian_json,
    test_aqua_debian2_json,
    test_aqua_debian3_json,
    test_aqua_debian4_json,
    test_aqua_debian5_json,
    test_aqua_debian6_json,
    test_aqua_debian7_json,
):
    aqualatest = AquaSource()

    # alsa
    cve_data = aqualatest.convert(test_aqua_alsa_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1

    # alas
    cve_data = aqualatest.convert(test_aqua_alas_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 7
    assert cve_index_count == 7
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-34903")))
    assert results_count == 7

    # rlsa
    cve_data = aqualatest.convert(test_aqua_rlsa_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 4
    assert cve_index_count == 4
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-32746")))
    assert results_count == 4

    # ubuntu
    cve_data = aqualatest.convert(test_aqua_ubuntu_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 27
    assert cve_index_count == 27
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-45406")))
    assert results_count == 27

    # ubuntu1
    cve_data = aqualatest.convert(test_aqua_ubuntu1_json)
    assert cve_data
    assert len(cve_data) == 8

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 8
    assert cve_index_count == 8
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-6083")))
    assert results_count == 8

    # ubuntu2
    cve_data = aqualatest.convert(test_aqua_ubuntu2_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 10
    assert cve_index_count == 10
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-3715")))
    assert results_count == 10

    # ubuntu3
    cve_data = aqualatest.convert(test_aqua_ubuntu3_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 14
    assert cve_index_count == 14
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-3219")))
    assert results_count == 14

    # redhat
    cve_data = aqualatest.convert(test_aqua_redhat_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 14
    assert cve_index_count == 14
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-45418")))
    assert results_count == 14

    # redhat2
    cve_data = aqualatest.convert(test_aqua_redhat2_json)
    assert cve_data
    assert len(cve_data) == 7

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 7
    assert cve_index_count == 7
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-21824")))
    assert results_count == 7

    # arch
    cve_data = aqualatest.convert(test_aqua_arch_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2019-13045")))
    assert results_count == 1

    # opensuse
    cve_data = aqualatest.convert(test_aqua_opensuse_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-29869")))
    assert results_count == 1

    # suse
    cve_data = aqualatest.convert(test_aqua_suse_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2021-3618")))
    assert results_count == 1

    # suse1
    cve_data = aqualatest.convert(test_aqua_suse_json1)
    assert not cve_data

    # photon
    cve_data = aqualatest.convert(test_aqua_photon_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2021-3618")))
    assert results_count == 1

    # debian
    cve_data = aqualatest.convert(test_aqua_debian_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 4
    assert cve_index_count == 4
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2019-18625")))
    assert results_count == 4

    # debian2
    cve_data = aqualatest.convert(test_aqua_debian2_json)
    assert not cve_data

    # debian3
    cve_data = aqualatest.convert(test_aqua_debian3_json)
    assert cve_data

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-3567")))
    assert results_count == 1

    # debian4
    cve_data = aqualatest.convert(test_aqua_debian4_json)
    assert not cve_data

    # debian5
    cve_data = aqualatest.convert(test_aqua_debian5_json)
    assert cve_data
    assert len(cve_data) == 2

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 2
    assert cve_index_count == 2
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2021-22890")))
    assert results_count == 2

    # debian6
    cve_data = aqualatest.convert(test_aqua_debian6_json)
    assert cve_data
    assert len(cve_data) == 2

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 2
    assert cve_index_count == 2
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-32091")))
    assert results_count == 2

    # debian7
    cve_data = aqualatest.convert(test_aqua_debian7_json)
    assert cve_data
    assert len(cve_data) == 1

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 1
    assert cve_index_count == 1
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2020-35448")))
    assert results_count == 1


def test_ubuntu_convert(test_aqua_ubuntu4_json, test_aqua_ubuntu5_json):
    aqualatest = AquaSource()

    # ubuntu4
    cve_data = aqualatest.convert(test_aqua_ubuntu4_json)
    assert cve_data
    assert len(cve_data) == 14

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 14
    assert cve_index_count == 14
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-32081")))
    assert results_count == 14

    # ubuntu5
    cve_data = aqualatest.convert(test_aqua_ubuntu5_json)
    assert cve_data
    assert len(cve_data) == 10

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 10
    assert cve_index_count == 10
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-3821")))
    assert results_count == 10


def test_wolfi_convert(test_aqua_cg_json, test_aqua_wolfi_json):
    aqualatest = AquaSource()

    cve_data = aqualatest.convert(test_aqua_cg_json)
    assert cve_data
    assert len(cve_data) == 9

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 9
    assert cve_index_count == 9
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2022-0543")))
    assert results_count == 1

    cve_data = aqualatest.convert(test_aqua_wolfi_json)
    assert cve_data
    assert len(cve_data) == 2

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 2
    assert cve_index_count == 2
    results_count = len(list(search.search_by_any("CVE-2020-8022")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2021-3121")))
    assert results_count == 1


def test_alpine_convert(test_aqua_alpine_unfixed_json):
    aqualatest = AquaSource()

    cve_data = aqualatest.convert(test_aqua_alpine_unfixed_json)
    assert cve_data
    assert len(cve_data) == 3

    db6.clear_all()
    aqualatest.store(cve_data)
    cve_data_count, cve_index_count = db6.stats()
    assert cve_data_count == 3
    assert cve_index_count == 3
    results_count = len(list(search.search_by_any("CVE-2024-25451")))
    assert results_count == 0
    results_count = len(list(search.search_by_any("CVE-2024-25450")))
    assert results_count == 3


def test_vuln_location():
    vl = VulnerabilityLocation.from_values(
        "cpe:2.3:a:pivotal_software:spring_framework:3.2.0:*:*:*:*:*:*:*",
        "3.2.0",
        "3.2.0",
        "",
        "",
    )
    assert vl.version == "3.2.0"
    vl = VulnerabilityLocation.from_values(
        "cpe:2.3:a:org.springframework:spring-web:*:*:*:*:*:*:*:*",
        "5.0.0.RC2",
        "*",
        "",
        "5.0.0.RC3",
    )
    assert vl.version == ">=5.0.0.RC2-<5.0.0.RC3"



from vdb.lib import utils as utils


def test_normalise():
    res = utils.normalise_num(100, 3)
    assert res == 100
    res = utils.normalise_num(100, 4)
    assert res == 1000
    res = utils.normalise_num(100, 2)
    assert res == 100
    res = utils.normalise_num('5.4b1', 3)
    assert res == 541
    assert utils.normalise_version_str("2.0.0", 3) == "2.0.0"
    assert utils.normalise_version_str("2.0.10", 4) == "2.0.10.0"


def test_version_compare():
    res = utils.version_compare("2.0.0", "2.0.0", "3.0.0", "2.0.0", "3.0.0")
    assert not res
    res = utils.version_compare("3.0.0", "2.0.0", "3.0.0", "2.0.0", "3.0.0")
    assert not res
    res = utils.version_compare("3.0.0", "2.0.0", "2.7.9.4")
    assert not res
    res = utils.version_compare("2.0.0", "2.0.0", "2.7.9.4")
    assert res
    res = utils.version_compare("2.7.0", "2.0.0", "2.7.9.4")
    assert res
    res = utils.version_compare("2.7.9.4", "2.0.0", "2.7.9.4")
    assert res
    res = utils.version_compare("2.7.9.5", "2.0.0", "2.7.9.4")
    assert not res
    res = utils.version_compare("1.0.0", "2.0.0", "2.7.9.4")
    assert not res
    res = utils.version_compare("4.0.0", "2.0.0", "2.7.9.4")
    assert not res
    res = utils.version_compare("3.7.9.4", "2.0.0", "*")
    assert res
    res = utils.version_compare("2.10.0", "2.1.0", "2.1.5")
    assert not res
    res = utils.version_compare("2.10.0", "2.0.0", "2.9.10")
    assert not res
    res = utils.version_compare("2.0.0", None, "2.7.9", "2.0.0")
    assert not res
    res = utils.version_compare("2.7.9", "2.0.0", None, None, "2.7.9")
    assert not res
    res = utils.version_compare("1.0", "1.0", "3.0")
    assert res
    res = utils.version_compare("1.0", "1.0.0", "3.0")
    assert res
    res = utils.version_compare("2.0", "1.0", "3.0")
    assert res
    res = utils.version_compare("2.0.0", "1.0", "3.0")
    assert res
    res = utils.version_compare("3.0", "1.0", "3.0")
    assert res
    res = utils.version_compare("3.0.0", "1.0", "3.0")
    assert res
    res = utils.version_compare("4.0", "1.0", "3.0")
    assert not res
    res = utils.version_compare("2.9.10.5", "*", "*", None, "2.8.11")
    assert not res
    res = utils.version_compare("2.9.10.5", "2.0.0", "*", None, "2.9.10.5")
    assert not res
    res = utils.version_compare("2.9.10.3", "2.0.0", "*", None, "2.9.10.5")
    assert res
    res = utils.version_compare("2.9.10.4", "2.0.0", "*", None, "2.9.10.5")
    assert res
    res = utils.version_compare("2.9.10.3", "2.9.0", "2.9.10.4", None, None)
    assert res
    res = utils.version_compare("2.9.10.4", "2.9.0", "2.9.10.4", None, None)
    assert res
    res = utils.version_compare("2.9.10.3", None, "2.9.10.4")
    assert res
    res = utils.version_compare("2.9.10.4", None, "2.9.10.5")
    assert res
    res = utils.version_compare("2.9.10.4", None, "2.9.10.1")
    assert not res
    res = utils.version_compare("2.9.10.4", None, "2.9.10")
    assert not res
    res = utils.version_compare("2.9.10.3", "0", "2.9.10.4")
    assert res
    res = utils.version_compare("2.9.10.4", "0", "2.9.10.5")
    assert res
    res = utils.version_compare("2.9.10.4", "0", "2.9.10.1")
    assert not res
    res = utils.version_compare("2.9.10.4", "0", "2.9.10")
    assert not res
    res = utils.version_compare("5.2.0.RELEASE", "5.2.0", "5.2.4")
    assert res
    res = utils.version_compare("5.2.1.FINAL", "5.2.0", "5.2.4")
    assert res
    res = utils.version_compare("5.2.5.FINAL", "5.2.0", "5.2.4")
    assert not res
    res = utils.version_compare("5.2.0.RELEASE", "5.2.0-alpha1", "5.2.4")
    assert res
    res = utils.version_compare("5.2.1.FINAL", "5.2.0-beta.1", "5.2.4")
    assert res
    res = utils.version_compare("5.2.5.FINAL", "5.2.0", "5.2.4")
    assert not res
    res = utils.version_compare("2.0.27.Final", None, "2.1.0")
    assert res
    res = utils.version_compare("2.0.27.Final", None, "2.0.29")
    assert res
    res = utils.version_compare("2.0.27.Final", None, None, None, "2.1.0")
    assert res
    res = utils.version_compare("2.0.27.Final", None, None, None, "2.0.29")
    assert res
    res = utils.version_compare("2.0.27.Final", "*", "*", None, "2.1.1")
    assert res
    res = utils.version_compare("2.0.27.Final", "*", "*", None, "2.0.29")
    assert res
    res = utils.version_compare("2.0.27.RELEASE", "*", "2.0.27", None, None)
    assert res
    res = utils.version_compare("0.6.18", "0", "0.6.19-r0", None, None)
    assert res
    res = utils.version_compare("2.15.4-r0", "0", "2.14.1-r0", None, None)
    assert not res
    res = utils.version_compare("1.8.19", "0", "1.8.20_p2-r0", None, None)
    assert res
    res = utils.version_compare("5.8.9", "0", "6.0_p20170701-r0", None, None)
    assert res
    res = utils.version_compare("1.5", "0", "1.6_rc2-r5", None, None)
    assert res
    res = utils.version_compare("0.99.1", "0", "0.99.4-r0", None, None)
    assert res
    res = utils.version_compare("0.7.0", "0", "0.7.1_alpha-r0", None, None)
    assert res
    res = utils.version_compare("0.7.0", "0", "*", None, None)
    assert res
    res = utils.version_compare("1.0.3", "0", "*", None, None)
    assert res
    res = utils.version_compare("1.0.3", "1.4.1", "*", None, None)
    assert not res
    res = utils.version_compare("12.1.0.2.0", "*", "12.1.0.2", None, None)
    assert res


def test_version_compare_go():
    res = utils.version_compare("v1.1.1", "v1.1.0", "v1.1.2")
    assert res
    res = utils.version_compare("v1.1.3", "v1.1.0", "v1.1.2")
    assert not res
    res = utils.version_compare(
        "v0.0.0-20190308221718-c2843e01d9a2", "2019-03-25", "2019-03-25"
    )
    assert not res
    res = utils.version_compare("v0.0.0-20190308221718-c2843e01d9a2", "*", "2017-03-17")
    assert not res
    res = utils.version_compare("v0.0.0-20190308221718-c2843e01d9a2", "*", "2019-03-17")
    assert res
    res = utils.version_compare(
        "v0.0.0-20180904163835-0709b304e793", "2019-03-25", "2019-03-25"
    )
    assert not res
    res = utils.version_compare(
        "v0.0.0-20180904163835-0709b304e793", "2018-09-03", "2018-09-05"
    )
    assert res
    res = utils.version_compare(
        "v0.0.0-20180904163835-0709b304e793", "2018-09-04", "2018-09-05"
    )
    assert res
    res = utils.version_compare(
        "v0.0.0-20180904163835-0709b304e793", "2018-09-05", "2018-09-05"
    )
    assert not res
    res = utils.version_compare("v0.0.0-20180904163835-0709b304e793", "*", "2017-03-17")
    assert not res
    res = utils.version_compare("v0.0.0-20180826012351-8a410e7b638d", "*", "2018-09-25")
    assert res
    res = utils.version_compare("v0.0.0-20180826012351-8a410e7b638d", "*", "2018-07-12")
    assert not res
    res = utils.version_compare("v0.0.0-20180826012351-8a410e7b638d", "*", "2018-09-27")
    assert res


def test_version_parts_compare():
    res = utils.version_compare("42.2.8", "*", "9.1.22")
    assert not res
    res = utils.version_compare("42.2.8", "*", "9.3.15")
    assert not res
    res = utils.version_compare("42.2.8", "*", "9.0")
    assert not res
    res = utils.version_compare("1.2.0", "0.7.0", "1.2.0")
    assert res
    res = utils.version_compare("2.1.8", "2.0.0", "2.0.14")
    assert not res
    res = utils.version_compare("2.1.800.0", "2.0.0", "2.2.14")
    assert res
    res = utils.version_compare("2.1.8.0", "2.0.0.800", "2.2.14.10")
    assert res
    res = utils.version_compare("82.1.8.0", "52.0.0", "96.2")
    assert res


def test_version_build_compare():
    res = utils.version_compare("6.0.1", "3.01", "5.4b2", None, None)
    assert not res
    res = utils.version_compare("6.0.1", "5.4", "5.4b2", None, None)
    assert not res
    res = utils.version_compare("6.0.1", "5.4b1", "5.4b4", None, None)
    assert not res
    res = utils.version_compare("6.0.1", "2.11", "5.8b1", None, None)
    assert not res
    res = utils.version_compare("6.0.1", "2.11", "6.8b1", None, None)
    assert res
    res = utils.version_compare("6.0.1", "2.11", "6.8.0b1", None, None)
    assert res
    res = utils.version_compare("1.2.0", "1.2.0-alpha", "1.2.0-beta")
    assert not res
    res = utils.version_compare("1.2.0", "1.1.0-alpha", "1.2.0-beta")
    assert not res
    res = utils.version_compare("1.3.0", "1.2.0-alpha", "1.3.1-beta")
    assert res
    res = utils.version_compare("1.3.0", "1.2.0", "1.3.1-beta")
    assert res
    res = utils.version_compare("1.3.0", "1.2.0-beta", "1.3.2.0")
    assert res
    res = utils.version_compare("4.3.6.RELEASE", "5.0.0.RC1", "5.0.0.RC3")
    assert not res


def test_version_build_compare1():
    sver, srest = utils.convert_to_semver("2.1.5.1-r3")
    assert sver
    res = utils.version_compare("2.1.5.1-r3", "1.2.1-r0", "2.1.5.1-r1")
    assert not res
    res = utils.version_compare("2.1.5.1-r3", "2.1.5.1-r2", "2.1.5.1-r6")
    assert res
    res = utils.version_compare("2.1.5.1-rc3", "2.1.5.1-rc2", "2.1.5.1-rc6")
    assert res


def test_version_build_diff_compare():
    res = utils.version_compare("7.0.0", "*", "*", None, "2020-04-23t00-58-49z")
    assert not res
    res = utils.version_compare("7.0.0", "*", "*", None, "2018-05-16t23-35-33z")
    assert not res
    res = utils.version_compare("7.0.0", "2018-05-16t23-35-33z", "2020-04-23t00-58-49z")
    assert not res
    res = utils.version_compare(
        "7.0.0", None, "2020-04-23t00-58-49z", "2020-04-23t00-58-49z", None
    )
    assert not res
    res = utils.version_compare(
        "7.0.0", None, "2018-05-16t23-35-33z", "2018-05-16t23-35-33z", None
    )
    assert not res
    res = utils.version_compare("7.0.0", "*", None, "2020-04-23t00-58-49z", None)
    assert not res
    res = utils.version_compare("7.0.0", "*", None, "2018-05-16t23-35-33z", None)
    assert not res


def test_version_hash_compare():
    res = utils.version_compare(
        "3.1.2", "0", None, None, "acb672b6a179567632e032f547582f30fa2f4aa7"
    )
    assert not res
    res = utils.version_compare(
        "3.1.2",
        "acb672b6a179567632e032f547582f30fa2f4aa7",
        "acb672b6a179567632e032f547582f30fa2f4aa7",
        None,
        None,
    )
    assert not res
    res = utils.version_compare(
        "8b626d45127d6f5ada7d815b83cfdc09e8cb1394",
        "8b626d45127d6f5ada7d815b83cfdc09e8cb1394",
        "8b626d45127d6f5ada7d815b83cfdc09e8cb1394",
        None,
        None,
    )
    assert res


def test_os_build_compare():
    res = utils.version_compare("2.8.4-1+squeeze4", "0", "2.8.6-1+squeeze4")
    assert res
    res = utils.version_compare("2.8.4-7+squeeze4", "0", "2.8.6-1+squeeze4")
    assert res
    res = utils.version_compare("2.6.34-1squeeze8", "0", "2.6.32-48squeeze8")
    assert not res
    res = utils.version_compare("3.0.1-8+wheezy6", "0", "3.0.4-3+wheezy6")
    assert res


def test_os_build_compare1():
    res = utils.version_compare("1:51.2.1-1~deb7u1", "0", "1:52.2.1-1~deb7u1")
    assert res
    res = utils.version_compare("2:51.2.1-1~deb7u1", "0", "1:52.2.1-1~deb7u1")
    assert res
    res = utils.version_compare("6:51.2.1-1~deb7u1", "0", "2:52.2.1-1~deb7u1")
    assert res
    res = utils.version_compare(
        "1.2.0-1.2+wheezy4+deb7u1", "0", "1.2.1-2.2+wheezy4+deb7u1"
    )
    assert res
    res = utils.version_compare("8:7.2.947-7+deb7u4", "0", "2:7.3.547-7+deb7u4")
    assert res
    res = utils.version_compare("9.04~dfsg-6.3+deb7u7", "0", "9.05~dfsg-6.3+deb7u7")
    assert res
    res = utils.version_compare("1:1.7.5.4-1+wheezy5", "0", "1:1.7.10.4-1+wheezy5")
    assert res
    res = utils.version_compare("1:1.7.11.1-1+wheezy5", "0", "1:1.7.10.4-1+wheezy5")
    assert not res
    res = utils.version_compare(
        "7u180-2.6.14-2~deb8u1", "7u179-2.6.14-2~deb8u1", "7u181-2.6.14-2~deb8u1"
    )
    assert res
    res = utils.version_compare(
        "7u182-2.8.14-2~deb8u1", "7u179-2.6.14-2~deb8u1", "7u181-2.6.14-2~deb8u1"
    )
    assert res
    res = utils.version_compare(
        "1.0.8~git20140621.1.440916e+dfsg1-13~deb8u3",
        "0",
        "1.1.0~git20140921.1.440916e+dfsg1-13~deb8u3",
    )
    assert res
    res = utils.version_compare("2:3.25-1+debu8u4", "0", "2:3.26-1+debu8u4")
    assert res
    res = utils.version_compare(
        "2.4.9", "0", "2.5.0.26054~ReleaseCandidate3.ds2-1+squeeze1"
    )
    assert res
    res = utils.version_compare("2.4.15-7.woody.2.1", "0", "2.4.17-2.woody.2.2")
    assert res
    res = utils.version_compare(
        "0.12.0+git20120207.aaa852f-1+deb7u1",
        "0",
        "0.12.1+git20120407.aaa852f-1+deb7u1",
    )
    assert res
    res = utils.version_compare("2.36.1-8+deb11u1", 0, "*", None, "2.36.1-8+deb11u1")
    assert not res
    res = utils.version_compare("1.10-4+deb11u1", 0, "*", None, "1.10-4+deb11u1")
    assert not res
    res = utils.version_compare("1.15.0", 0, "*", None, "1.15.1-2ubuntu2.2")
    assert res
    res = utils.version_compare("1:1.1.1k-7.el8_6", 0, "*", None, "1.1.1k")
    assert not res
    res = utils.version_compare("1.1.1f-1ubuntu2.8", 0, "*", None, "1.1.1-1ubuntu2.1")
    assert not res
    res = utils.version_compare("0.23.22-1.amzn2.0.1", 0, "*", None, "0.23.22")
    assert not res


def test_ubuntu_openssl():
    res = utils.version_compare("1.1.1f-1ubuntu2.8", 0, "*", None, "1.1.1.e")
    assert not res
    res = utils.version_compare(
        "1.1.1f-1ubuntu2.8", 0, "*", None, "1.1.1-1ubuntu2.1~18.04.2"
    )
    assert not res
    res = utils.version_compare(
        "1.1.1f-1ubuntu2.8", 0, "*", None, "1.1.1-1ubuntu2.1~18.04.6"
    )
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1g-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1k-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1d-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1j-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1i-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1l-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1a-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1~~pre9-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1o-1")
    assert res
    res = utils.version_compare("1.10-0ubuntu4", 0, "*", None, "1.10-0ubuntu4.1")
    assert res
    res = utils.version_compare("2.2.19-3ubuntu2.1", 0, "*", None, "*")
    assert res
    res = utils.version_compare("2.2.19-3ubuntu2.1", 0, "*", None, None)
    assert res
    res = utils.version_compare("2.34-0.1ubuntu9.1", 0, "*", None, "2.34-0.1ubuntu9.3")
    assert res
    res = utils.version_compare("3.0-1", 0, "*", None, "3.0-1ubuntu0.1")
    assert res
    # Not working
    # res = utils.version_compare("1.1.0j-1~deb9u1", 0, "*", None, "1.1.0f-3+deb9u1")
    # assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1g-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", 0, "*", None, "1.1.1k-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", "*", "*", None, "1.1.1g-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb11u3", "*", "*", None, "1.1.1k-1")
    assert not res
    res = utils.version_compare("1.1.1n-0+deb10u3", "*", "*", None, "1.1.1d-1")
    assert not res
    res = utils.version_compare("2:8.39-12build1", 0, "*", None, "8.39-9ubuntu0.1")
    assert not res
    res = utils.version_compare(
        "7.7.0+dfsg-1ubuntu1", 0, "*", None, "7.7.0+dfsg-1ubuntu1.1"
    )
    assert res


def test_ubuntu_compare2():
    # res = utils.version_compare("1.45.5-2ubuntu1", 0, "*", None, "1.45.5-1")
    # assert not res
    res = utils.version_compare("3.0-1", 0, "*", None, "3.0-1ubuntu0.1")
    assert res
    res = utils.version_compare("10.34-7", 0, "*", None, "10.34-7ubuntu0.1")
    assert res
    res = utils.version_compare(
        "2:6.2.0+dfsg-4", 0, "*", None, "2:6.2.0+dfsg-4ubuntu0.1"
    )
    assert res
    # res = utils.version_compare(
    #     "3.8.10-0ubuntu1~20.04.6", 0, "*", None, "3.8.10-0ubuntu1~20.04.1"
    # )
    # assert not res
    # res = utils.version_compare(
    #     "20210119~20.04.2", 0, "*", None, "20211016ubuntu0.20.04.1"
    # )
    # assert not res
    # res = utils.version_compare("2.1.27+dfsg-2", 0, "*", None, "2.1.27+dfsg-2ubuntu0.1")
    # assert res


def test_debian_build_compare():
    res = utils.version_compare("1:2019.10.06-1", 0, "*", None, "20070829-6+deb7u1")
    assert not res
    res = utils.version_compare("1.0.2l-2+deb9u3", 0, "*", None, "1.0.2u-1~deb9u5")
    assert res
    res = utils.version_compare("232-25+deb9u4", 0, "*", None, "232-25+deb9u8")
    assert res
    res = utils.version_compare("1.1.0f-3+deb9u2", 0, "*", None, "1.1.0h-3")
    assert res
    res = utils.version_compare("4.9.110-3+deb9u4", 0, "*", None, "4.9.240-1")
    assert res
    res = utils.version_compare("232-25+deb9u4", 0, "*", None, "232-25+deb9u8")
    assert res
    res = utils.version_compare("3.5.8-5+deb9u3", 0, "*", None, "3.5.8-5+deb9u6")
    assert res
    res = utils.version_compare("1.43.4-2", 0, "*", None, "1.46.6~rc1-1")
    assert res
    res = utils.version_compare("1.0.2l-2+deb9u3", 0, "*", None, "1.1.0c-1")
    assert res
    res = utils.version_compare("4.9.110-3+deb9u4", 0, "*", None, "4.9.161-1")
    assert res
    res = utils.version_compare("4.9.110-3+deb9u4", 0, "*", None, "4.9.240-1")
    assert res
    res = utils.version_compare("4.9.110-3+deb9u4", 0, "*", None, "4.9.110-1")
    assert not res


def test_debian_build_compare2():
    res = utils.version_compare("4.9.110-3+deb9u4", 0, "*", None, "4.9.20-1")
    assert not res
    res = utils.version_compare("1.43.4-2", 0, "*", None, "1.46.6~rc1-1")
    assert res
    res = utils.version_compare("1.0.6-8.1", 0, "*", None, "1.0.6-9.1")
    assert res
    res = utils.version_compare("1.0.2l-2+deb9u3", 0, "*", None, "1.0.2l-2+deb9u2")
    assert not res
    res = utils.version_compare("7.52.1-5+deb9u7", 0, "*", None, "7.52.1-5+deb9u3")
    assert not res
    res = utils.version_compare("1.0.2l-2+deb9u3", "*", "*", None, "1.0.2l-2+deb9u2")
    assert not res
    res = utils.version_compare("7.52.1-5+deb9u7", "*", "*", None, "7.52.1-5+deb9u3")
    assert not res
    res = utils.version_compare("7.52.1-5+deb9u7", 0, "*", None, "7.52.1-5+deb9u2")
    assert not res
    res = utils.version_compare("1.7.6-2+deb9u3", 0, "*", None, "1.7.6-2+deb9u2")
    assert not res
    res = utils.version_compare("2.24-11+deb9u3", 0, "*", None, "2.24-11+deb9u2")
    assert not res
    res = utils.version_compare("4.9.110-3+deb9u4", 0, "*", None, "4.19.37-2")
    assert res
    res = utils.version_compare("4.9.110-3+deb9u4", 0, "*", None, "4.17.3-1")
    assert res
    res = utils.version_compare("4.9.110-3+deb9u4", 0, "*", None, "4.16.16-1")
    assert res
    res = utils.version_compare("1.15-1+deb9u1", 0, "*", None, "1.2.4-5woody4")
    assert not res
    res = utils.version_compare("1.15-1+deb9u1", 0, "*", None, "1.8.3+dfsg-4squeeze5")
    assert not res
    res = utils.version_compare(
        "1.15-1+deb9u1", 0, "*", None, "1.6.dfsg.4~beta1-5lenny7"
    )
    assert not res
    res = utils.version_compare("1.15-1+deb9u1", 0, "*", None, "1.10.1+dfsg-5+deb7u6")
    assert not res
    res = utils.version_compare("2018e-0+deb9u1", 0, "*", None, "2018c-0+deb7u1")
    assert not res


def test_debian_build_compare3():
    res = utils.version_compare("20161130+nmu1+deb9u1", 0, "*", None, "20200601~deb9u2")
    assert res
    res = utils.version_compare("232-25+deb9u4", 0, "*", None, "249~rc2-1")
    assert res
    res = utils.version_compare("232-25+deb9u4", 0, "*", None, "232-25+deb9u6")
    assert res
    res = utils.version_compare("232-25+deb9u4", 0, "*", None, "232-25+deb9u8")
    assert res
    res = utils.version_compare("232-25+deb9u4", "*", "*", None, "232-25+deb9u6")
    assert res
    res = utils.version_compare("232-25+deb9u4", "*", "*", None, "232-25+deb9u8")
    assert res
    res = utils.version_compare("2.50.3-2", 0, "*", None, "2.50.3-2+deb9u1")
    assert res
    res = utils.version_compare("2.35.2-2", 0, "*", None, "2.37-3")
    assert res
    res = utils.version_compare("2.35.2-2", "*", "*", None, "2.37-3")
    assert res
    res = utils.version_compare("2.36", "*", "*", None, "2.37-3")
    assert res
    res = utils.version_compare("247.3-7", "*", "*", None, "249~rc2-1")
    assert res
    res = utils.version_compare("247.3-7", 0, "*", None, "249~rc2-1")
    assert res
    res = utils.version_compare("247.3-7", "*", "*", None, "99.99.9")
    assert res
    res = utils.version_compare("4.3.0", "2.0.20126.16343", "4.3.3", None, None)
    assert res


def test_redhat_build_compare():
    data = """
1.18.2-21.el8 1.18.2-22.el8_7
1.11.948 1.12.261
2.12.1 2.12.6.1, 2.13.2.1
2.12.1 2.12.7.1, 2.13.4.1
2.12.1 2.12.7.1, 2.13.4
2.8.6 2.8.9
25.1-android 30.0
1.25.0 1.31.0
1.25.0 1.33.3
3.16.1 3.16.3, 3.19.6, 3.20.3, 3.21.7
2.6 2.7
1.10.9 1.9.16, 1.10.11
1.10.9 1.9.16, 1.10.11
6.5.0 6.6.0
1.20 1.21
1.20 1.21
1.20 1.21
1.20 1.21
1.9 1.10.0
2.3.0 2.5.1
1.5.21 1.6.0
1.14.3 1.15.3
2.13.3 2.13.9
1.28 1.31
1.28 1.31
1.28 1.31
1.28 1.31
1.28 1.32
1.30 1.31
1.30 1.31
1.30 1.31
1.30 1.31
1.30 1.32
6.7.1 11.8.5, 12.1.0
1.0.0 4.0.1, 3.0.1
1.7.1 2.4.0
1.25.11 1.26.5
v0.0.0-20210421170649-83a5a9bb288b 0.0.0-20211202192323-5770296d904e
v0.0.0-20210421170649-83a5a9bb288b 0.0.0-20220314234659-1baeb1ce4c0b
v0.0.0-20210326060303-6b1517762897 0.0.0-20210520170846-37e1c6afe023
v0.0.0-20210326060303-6b1517762897 0.0.0-20211209124913-491a49abca63
v0.0.0-20210326060303-6b1517762897 0.0.0-20220906165146-f3363e06e74c
v0.0.0-20210326060303-6b1517762897 0.0.0-20210428140749-89ef3d95e781
v0.0.0-20210502180810-71e4cd670f79 0.0.0-20220412211240-33da011f77ad
v0.0.0-20200622213623-75b288015ac9 0.0.0-20201216223049-8b5274cf687f
v0.0.0-20200622213623-75b288015ac9 0.0.0-20211202192323-5770296d904e
v0.0.0-20200622213623-75b288015ac9 0.0.0-20220314234659-1baeb1ce4c0b
v0.0.0-20210520170846-37e1c6afe023 0.0.0-20211209124913-491a49abca63
v0.0.0-20210520170846-37e1c6afe023 0.0.0-20220906165146-f3363e06e74c
v0.0.0-20210630005230-0f9fa26af87c 0.0.0-20220412211240-33da011f77ad
v0.3.6 0.3.7
v0.3.6 0.3.8
v0.2.0 v0.3.1
v1.11.0 1.11.1
v0.0.0-20210921065528-437939a70204 0.0.0-20220412211240-33da011f77ad
v0.0.0-20211019181941-9d821ace8654 0.0.0-20220412211240-33da011f77ad
v0.0.0-20210421170649-83a5a9bb288b 0.0.0-20211202192323-5770296d904e
v0.0.0-20210421170649-83a5a9bb288b 0.0.0-20220314234659-1baeb1ce4c0b
v0.0.0-20210405180319-a5a99cb37ef4 0.0.0-20210520170846-37e1c6afe023
v0.0.0-20210405180319-a5a99cb37ef4 0.0.0-20211209124913-491a49abca63
v0.0.0-20210405180319-a5a99cb37ef4 0.0.0-20220906165146-f3363e06e74c
v0.0.0-20210405180319-a5a99cb37ef4 0.0.0-20210428140749-89ef3d95e781
v0.0.0-20210616094352-59db8d763f22 0.0.0-20220412211240-33da011f77ad
v0.3.7 0.3.8
"""
    for line in data.split("\n"):
        tmp_a = line.split(" ")
        if len(tmp_a) != 2:
            continue
        res = utils.version_compare(tmp_a[0], 0, "*", None, tmp_a[1])
        assert res
        res = utils.version_compare(tmp_a[1], 0, "*", None, tmp_a[0])
        assert not res
    res = utils.version_compare("14.20.1", 0, "*", None, "12")
    assert not res
    res = utils.version_compare("6.0", 0, "*", None, "5.4")
    assert not res
    res = utils.version_compare("3.3.2", "1.8", None, None, "41.0.2")
    assert res
    res = utils.version_compare("39.0.0", "0.8.1", None, None, "39.0.1")
    assert res
    res = utils.version_compare("39.0.0", "0", None, None, "1.5.3")
    assert not res
    res = utils.version_compare("38.0.0", "37.0.0", "*", None, "38.0.3")
    assert res
    res = utils.version_compare("3.3.0", "0.5", "*", None, "41.0.0")
    assert res
    res = utils.version_compare("41.0.0", "40.0.0", "*", None, "41.0.2")
    assert res


def test_parse_uri():
    vendor, package, version, cve_type = utils.parse_cpe(
        "cpe:2.3:o:google:android:9.0:*:*:*:*:*:*:*"
    )
    assert vendor == "google"
    assert package == "android"
    assert version == "9.0"


def test_version_len():
    assert utils.version_len("1.0.0") == 3
    assert utils.version_len("2.1.800.5") == 6
    assert utils.version_len("1.2.0-beta1") == 3
    assert utils.version_len("1.3.0.beta1") == 3


def test_fix_test():
    assert (
        utils.fix_text(
            "Unauthenticated crypto and weak IV in Magento\\Framework\\Encryption"
        )
        == "Unauthenticated crypto and weak IV in Magento Framework Encryption"
    )


def test_convert_md_references():
    assert utils.convert_md_references(
        "- [Issue #60](https://github.com/tj/node-growl/issues/60)\n- [PR #61](https://github.com/tj/node-growl/pull/61)"
    ) == [
        {"name": "Issue #60", "url": "https://github.com/tj/node-growl/issues/60"},
        {"name": "PR #61", "url": "https://github.com/tj/node-growl/pull/61"},
    ]


def test_ubuntu_versions():
    assert utils.check_epoch("1:4.14-1.1+deb7u1build0.12.04.1")
    assert utils.convert_to_semver("0.3.14-11ubuntu6")[0]
    assert utils.convert_to_semver("1.4.9a-1ubuntu0.1")[0]
    assert utils.convert_to_semver("20140405-0ubuntu1")[0]
    assert utils.convert_to_semver("1.10+dfsg~beta1-2ubuntu0.7")[0]
    assert utils.convert_to_semver("1.11.3+dfsg-3ubuntu2")[0]
    assert utils.convert_to_semver("1.0.4cvs20051004-2")[0]
    assert utils.convert_to_semver("7.0.0-0ubuntu45")[0]
    assert utils.convert_to_semver("1.5.0+1.5.1cvs20051015-1ubuntu10")[0]
    assert utils.convert_to_semver("1.2.8rel-1ubuntu3")[0]
    assert utils.convert_to_semver("3.3.8really3.3.7-0ubuntu5.2")[0]
    assert utils.convert_to_semver("1.5.0+1.5.1cvs20051015-1ubuntu10")[0]
    assert utils.convert_to_semver("0.9.2+cvs.1.0.dev.2004.07.28-4ubuntu1")[0]
    assert utils.convert_to_semver("8.1.2-0.20040524cvs-2")[0]
    assert utils.convert_to_semver("6.2.4.5.dfsg1-0.14ubuntu0.1")[0]
    assert utils.convert_to_semver("1.5.dfsg+1.5.0.13~prepatch070731-0ubuntu1")[0]
    assert utils.convert_to_semver("01.03.00.99.svn.300-3")[0]
    assert utils.check_epoch("1:1.1.1k-7.el8_6")


def test_parse_purl():
    assert utils.parse_purl(
        "pkg:maven/org.jenkins-ci.plugins/JDK_Parameter_Plugin"
    ) == {
        "type": "maven",
        "namespace": "org.jenkins-ci.plugins",
        "name": "JDK_Parameter_Plugin",
        "version": None,
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.parse_purl("pkg:golang/github.com/sjqzhang/go-fastdfs") == {
        "type": "golang",
        "namespace": "",
        "name": "github.com/sjqzhang/go-fastdfs",
        "version": None,
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.parse_purl("pkg:golang/vitess.io/vitess") == {
        "type": "golang",
        "namespace": "",
        "name": "vitess.io/vitess",
        "version": None,
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.parse_purl("pkg:golang/stdlib") == {
        "type": "golang",
        "namespace": None,
        "name": "stdlib",
        "version": None,
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.parse_purl("pkg:golang/github.com/containerd/containerd") == {
        "type": "golang",
        "namespace": "",
        "name": "github.com/containerd/containerd",
        "version": None,
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.parse_purl("pkg:gem/remote:@https:%2F%2Frubygems.org%2F") == {}


def test_purl_vers_convert():
    assert (
        utils.to_purl_vers(
            "almalinux",
            [
                {
                    "version": "0",
                    "status": "affected",
                    "versionType": "semver",
                    "lessThan": "2.3.4",
                }
            ],
        )
        == "vers:rpm/<2.3.4"
    )
    assert (
        utils.to_purl_vers(
            "almalinux",
            [
                {
                    "version": "0",
                    "status": "affected",
                    "versionType": "semver",
                    "lessThan": "5.14.21-150500.13.5.1",
                }
            ],
        )
        == "vers:rpm/<5.14.21-150500.13.5.1"
    )
    assert (
        utils.to_purl_vers(
            "gem",
            [
                {
                    "version": "0.1.1",
                    "status": "affected",
                    "versionType": "gem",
                    "lessThanOrEqual": "4.3.5",
                }
            ],
        )
        == "vers:gem/>=0.1.1|<=4.3.5"
    )
    test_tuples = [
        (
            "npm",
            [
                {
                    "version": "0.0.0",
                    "status": "affected",
                    "versionType": "npm",
                    "lessThanOrEqual": "*",
                }
            ],
            "vers:npm/*",
        ),
        (
            "npm",
            [
                {
                    "version": "0.0.0",
                    "status": "affected",
                    "versionType": "npm",
                    "lessThan": "1.1.3",
                }
            ],
            "vers:npm/>=0.0.0|<1.1.3",
        ),
        (
            "npm",
            [
                {
                    "version": "0.0.0",
                    "status": "affected",
                    "versionType": "npm",
                    "lessThan": "3.5.0",
                }
            ],
            "vers:npm/>=0.0.0|<3.5.0",
        ),
        (
            "swift",
            [
                {
                    "version": "4.0.0-rc.2.5",
                    "status": "affected",
                    "versionType": "semver",
                    "lessThan": "4.29.4",
                }
            ],
            "vers:swift/>=4.0.0-rc.2.5|<4.29.4",
        ),
        (
            "swift",
            [
                {
                    "version": "0.0.0",
                    "status": "affected",
                    "versionType": "semver",
                    "lessThan": "4.60.3",
                }
            ],
            "vers:swift/>=0.0.0|<4.60.3",
        ),
        (
            "swift",
            [
                {
                    "version": "0.0.0",
                    "status": "affected",
                    "versionType": "semver",
                    "lessThan": "4.60.3",
                }
            ],
            "vers:swift/>=0.0.0|<4.60.3",
        ),
        (
            "maven",
            [
                {
                    "version": "8.5.0",
                    "status": "affected",
                    "versionType": "maven",
                    "lessThan": "8.5.94",
                }
            ],
            "vers:maven/>=8.5.0|<8.5.94",
        ),
        (
            "maven",
            [
                {
                    "version": "11.0.0-M1",
                    "status": "affected",
                    "versionType": "maven",
                    "lessThan": "11.0.0-M12",
                }
            ],
            "vers:maven/>=11.0.0-M1|<11.0.0-M12",
        ),
        (
            "debian",
            [{"version": "88.88.8", "status": "affected", "versionType": "maven"}],
            "vers:deb/88.88.8",
        ),
        (
            "debian",
            [
                {
                    "version": "53.5",
                    "status": "affected",
                    "versionType": "maven",
                    "lessThanOrEqual": "53.5",
                }
            ],
            "vers:deb/53.5",
        ),
    ]
    for tt in test_tuples:
        assert utils.to_purl_vers(tt[0], tt[1]) == tt[2]


def test_url_to_purl():
    assert utils.url_to_purl("https://github.com/owasp-dep-scan/dep-scan") == {
        "name": "dep-scan",
        "namespace": "owasp-dep-scan",
        "qualifiers": None,
        "subpath": None,
        "type": "github",
        "version": None,
    }
    assert utils.url_to_purl(
        "https://github.com/opennms/opennms/commit/8710463077c10034fcfa06556a98fb1a1a64fd0d"
    ) == {
        "name": "opennms",
        "namespace": "opennms",
        "qualifiers": None,
        "subpath": None,
        "type": "github",
        "version": "8710463077c10034fcfa06556a98fb1a1a64fd0d",
    }
    assert utils.url_to_purl(
        "https://git.eyrie.org/?p=kerberos/remctl.git%3ba=commit%3bh=86c7e44090c988112a37589d2c7a94029eb5e641"
    ) == {
        "type": "generic",
        "namespace": "git.eyrie.org/kerberos",
        "name": "remctl",
        "version": "86c7e44090c988112a37589d2c7a94029eb5e641",
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.url_to_purl(
        "https://git.savannah.gnu.org/cgit/wget.git/commit/?id=1fc9c95ec144499e69dc8ec76dbe07799d7d82cd"
    ) == {
        "type": "generic",
        "namespace": "git.savannah.gnu.org",
        "name": "wget",
        "version": "1fc9c95ec144499e69dc8ec76dbe07799d7d82cd",
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.url_to_purl(
        "https://git.gnupg.org/cgi-bin/gitweb.cgi?p=libgcrypt.git%3ba=commit%3bh=9010d1576e278a4274ad3f4aa15776c28f6ba965"
    ) == {
        "type": "generic",
        "namespace": "git.gnupg.org",
        "name": "libgcrypt",
        "version": "9010d1576e278a4274ad3f4aa15776c28f6ba965",
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.url_to_purl(
        "https://git.openssl.org/gitweb/?p=openssl.git%3ba=commitdiff%3bh=3984ef0b72831da8b3ece4745cac4f8575b19098"
    ) == {
        "type": "generic",
        "namespace": "git.openssl.org",
        "name": "openssl",
        "version": "3984ef0b72831da8b3ece4745cac4f8575b19098",
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.url_to_purl(
        "https://github.com/rsyslog/librelp/blob/532aa362f0f7a8d037505b0a27a1df452f9bac9e/src/tcp.c#l1205"
    ) == {
        "type": "github",
        "namespace": "rsyslog",
        "name": "librelp",
        "version": "532aa362f0f7a8d037505b0a27a1df452f9bac9e",
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.url_to_purl(
        "https://github.com/lightsaml/lightsaml/releases/tag/1.3.5"
    ) == {
        "type": "github",
        "namespace": "lightsaml",
        "name": "lightsaml",
        "version": "1.3.5",
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.url_to_purl(
        "https://gitlab.labs.nic.cz/labs/bird/commit/e8bc64e308586b6502090da2775af84cd760ed0d"
    ) == {
        "type": "generic",
        "namespace": "gitlab.labs.nic.cz/labs",
        "name": "bird",
        "version": "e8bc64e308586b6502090da2775af84cd760ed0d",
        "qualifiers": None,
        "subpath": None,
    }
    assert utils.url_to_purl(
        "http://git.kernel.org/cgit/linux/kernel/git/torvalds/linux.git/commit/?id=0fa3ecd87848c9c93c2c828ef4c3a8ca36ce46c7"
    ) == {
        "type": "generic",
        "namespace": "git.kernel.org/linux",
        "name": "kernel",
        "version": "0fa3ecd87848c9c93c2c828ef4c3a8ca36ce46c7",
        "qualifiers": None,
        "subpath": None,
    }



import os

from appdirs import user_cache_dir, user_data_dir

# NVD CVE json feed url
NVD_URL = "https://nvd.nist.gov/feeds/json/cve/1.1/nvdcve-1.1-%(year)s.json.gz"

# NVD start year. 2018 is quicker. 2002 is quite detailed but slow
NVD_START_YEAR = os.getenv("NVD_START_YEAR", "2018")
try:
    NVD_START_YEAR = int(NVD_START_YEAR)
except ValueError:
    pass

# GitHub advisory feed url
GHA_URL = os.getenv("GITHUB_GRAPHQL_URL", "https://api.github.com/graphql")

# No of pages to download from GitHub during a full refresh
GHA_PAGES_COUNT = os.getenv("GITHUB_PAGE_COUNT", "2")
NPM_PAGES_COUNT = os.getenv("NPM_PAGE_COUNT", "2")

# DB file dir
DATA_DIR = os.getenv("VDB_HOME", user_data_dir("vdb"))
if not os.path.exists(DATA_DIR):
    os.makedirs(DATA_DIR)

CACHE_DIR = os.getenv("VDB_CACHE", user_cache_dir("vdb"))
if not os.path.exists(CACHE_DIR):
    os.makedirs(CACHE_DIR)

# Binary db file
VDB_BIN_FILE = os.path.join(DATA_DIR, "data.vdb6")

# Binary DB index file
VDB_BIN_INDEX = os.path.join(DATA_DIR, "data.index.vdb6")

# NPM advisory url
NPM_SERVER = "https://registry.npmjs.org"
NPM_AUDIT_URL = NPM_SERVER + "/-/npm/v1/security/audits"
NPM_ADVISORIES_URL = NPM_SERVER + "/-/npm/v1/security/advisories"

NPM_APP_INFO = {"name": "appthreat-vdb", "version": "6.0.0"}

CVE_TPL = """
{"cve":{"data_type":"CVE","data_format":"MITRE","data_version":"4.0","CVE_data_meta":{"ID":"%(cve_id)s","ASSIGNER":"%(assigner)s"},"problemtype":{"problemtype_data":[{"description":[{"lang":"en","value":"%(cwe_id)s"}]}]},"references":{"reference_data": %(references)s},"description":{"description_data":[{"lang":"en","value":"%(description)s"}]}},"configurations":{"CVE_data_version":"4.0","nodes":[{"operator":"OR","cpe_match":[{"vulnerable":true,"cpe23Uri":"cpe:2.3:a:%(vendor)s:%(product)s:%(version)s:*:%(edition)s:*:*:*:*:*","versionStartExcluding":"%(version_start_excluding)s","versionEndExcluding":"%(version_end_excluding)s","versionStartIncluding":"%(version_start_including)s","versionEndIncluding":"%(version_end_including)s"}, {"vulnerable":false,"cpe23Uri":"cpe:2.3:a:%(vendor)s:%(product)s:%(fix_version_start_including)s:*:%(edition)s:*:*:*:*:*","versionStartExcluding":"%(fix_version_start_excluding)s","versionEndExcluding":"%(fix_version_end_excluding)s","versionStartIncluding":"%(fix_version_start_including)s","versionEndIncluding":"%(fix_version_end_including)s"}]}]},"impact":{"baseMetricV3":{"cvssV3":{"version":"3.1","vectorString":"%(vectorString)s","attackVector":"NETWORK","attackComplexity":"%(attackComplexity)s","privilegesRequired":"NONE","userInteraction":"%(userInteraction)s","scope":"UNCHANGED","confidentialityImpact":"%(severity)s","integrityImpact":"%(severity)s","availabilityImpact":"%(severity)s","baseScore":%(score).1f,"baseSeverity":"%(severity)s"},"exploitabilityScore":%(exploitabilityScore).1f,"impactScore":%(score).1f},"baseMetricV2":{"cvssV2":{"version":"2.0","vectorString":"AV:N/AC:M/Au:N/C:P/I:P/A:P","accessVector":"NETWORK","accessComplexity":"MEDIUM","authentication":"NONE","confidentialityImpact":"PARTIAL","integrityImpact":"PARTIAL","availabilityImpact":"PARTIAL","baseScore":%(score).1f},"severity":"%(severity)s","exploitabilityScore":%(exploitabilityScore).1f,"impactScore":%(score).1f,"acInsufInfo":false,"obtainAllPrivilege":false,"obtainUserPrivilege":false,"obtainOtherPrivilege":false,"userInteractionRequired":false}},"publishedDate":"%(publishedDate)s","lastModifiedDate":"%(lastModifiedDate)s"}
"""

osv_url_dict = {
    "javascript": "https://osv-vulnerabilities.storage.googleapis.com/npm/all.zip",
    "python": "https://osv-vulnerabilities.storage.googleapis.com/PyPI/all.zip",
    "go": "https://osv-vulnerabilities.storage.googleapis.com/Go/all.zip",
    "java": "https://osv-vulnerabilities.storage.googleapis.com/Maven/all.zip",
    "rust": "https://osv-vulnerabilities.storage.googleapis.com/crates.io/all.zip",
    "csharp": "https://osv-vulnerabilities.storage.googleapis.com/NuGet/all.zip",
    "ruby": "https://osv-vulnerabilities.storage.googleapis.com/RubyGems/all.zip",
    "dwf": "https://osv-vulnerabilities.storage.googleapis.com/DWF/all.zip",
    "gsd": "https://osv-vulnerabilities.storage.googleapis.com/GSD/all.zip",
    "hex": "https://osv-vulnerabilities.storage.googleapis.com/Hex/all.zip",
    "packagist": "https://osv-vulnerabilities.storage.googleapis.com/Packagist/all.zip",
    "pub": "https://osv-vulnerabilities.storage.googleapis.com/Pub/all.zip",
    "uvi": "https://osv-vulnerabilities.storage.googleapis.com/UVI/all.zip",
    "github": "https://osv-vulnerabilities.storage.googleapis.com/GitHub%20Actions/all.zip",
    "alpine": "https://osv-vulnerabilities.storage.googleapis.com/Alpine/all.zip",
    "debian": "https://osv-vulnerabilities.storage.googleapis.com/Debian/all.zip",
    "cran": "https://osv-vulnerabilities.storage.googleapis.com/CRAN/all.zip",
    "almalinux": "https://osv-vulnerabilities.storage.googleapis.com/AlmaLinux/all.zip",
    "rockylinux": "https://osv-vulnerabilities.storage.googleapis.com/Rocky%20Linux/all.zip",
    "swift": "https://osv-vulnerabilities.storage.googleapis.com/SwiftURL/all.zip",
    "git": "https://osv-vulnerabilities.storage.googleapis.com/GIT/all.zip",
}

# These feeds introduce too much false positives
if os.getenv("OSV_INCLUDE_FUZZ"):
    osv_url_dict[
        "linux"
    ] = "https://osv-vulnerabilities.storage.googleapis.com/Linux/all.zip"
    osv_url_dict[
        "oss-fuzz"
    ] = "https://osv-vulnerabilities.storage.googleapis.com/OSS-Fuzz/all.zip"
    osv_url_dict["android"] = "https://osv-vulnerabilities.storage.googleapis.com/Android/all.zip",

VULN_LIST_URL = "https://github.com/appthreat/vuln-list/archive/refs/heads/main.zip"

# Placeholder fix version to use to indicate max versions
PLACEHOLDER_FIX_VERSION = "99.99.9"

# Placeholder exclude version to use to indicate non-vulnerability
# This is highly important for debian where a specific distro may be non-vulnerable
# While CPEs are the correct method of representing the exclusion, this version hack is aimed to be a short workaround
PLACEHOLDER_EXCLUDE_VERSION = "88.88.8"

# How many CVEs should be packed and written to the db file as a unit
# A large value here requires a larger max_buffer_size. Else could lead to msgpack.exceptions.BufferFull exceptions during read
BATCH_WRITE_SIZE = 20

# Limits size of unpacked data
MAX_BUFFER_SIZE = 200 * 1024 * 1024  # 200 MiB

THREAT_TO_SEVERITY = {
    "unspecified": "LOW",
    "": "LOW",
    "negligible": "LOW",
    "low": "LOW",
    "unimportant": "LOW",
    "severity_low": "LOW",
    "medium": "MEDIUM",
    "severity_medium": "MEDIUM",
    "moderate": "MEDIUM",
    "severity_moderate": "MEDIUM",
    "important": "HIGH",
    "high": "HIGH",
    "severity_important": "HIGH",
    "critical": "CRITICAL",
    "severity_critical": "CRITICAL",
}

VENDOR_TO_VERS_SCHEME = {
    "almalinux": "rpm",
    "rocky": "rpm",
    "photon": "rpm",
    "ubuntu": "deb",
    "debian": "deb",
    "suse": "rpm",
    "redhat": "rpm",
    "opensuse": "rpm",
    "alpine": "apk",
    "gentoo": "ebuild",
    "amazon": "rpm",
    "wolfi": "apk",
    "chainguard": "apk",
}

OS_PKG_TYPES = (
    "deb",
    "apk",
    "rpm",
    "swid",
    "alpm",
    "docker",
    "oci",
    "container",
    "qpkg",
    "buildroot",
    "coreos",
    "ebuild",
)

# URL for the pre-compiled database
VDB_DATABASE_URL = os.getenv("VDB_DATABASE_URL", "ghcr.io/appthreat/vdbxz:v6")



# generated by datamodel-codegen:
#   filename:  CVE_JSON_5.0_schema.json
#   timestamp: 2024-02-07T18:22:42+00:00

from __future__ import annotations

from enum import Enum
from typing import Annotated, Optional

from pydantic import BaseModel, Field

from vdb.lib.cve_model.common import (
    AccessComplexityType,
    AccessVectorType,
    AuthenticationType,
    CiaRequirementType,
    CiaTypeModel,
    CollateralDamagePotentialType,
    ExploitabilityType,
    RemediationLevelType,
    ReportConfidenceType,
    ScoreType,
    TargetDistributionType,
)


class Version(Enum):
    field_2_0 = "2.0"


class Field0(BaseModel):
    version: Annotated[Version, Field(description="CVSS Version")]
    vectorString: Annotated[
        str,
        Field(
            pattern="^((AV:[NAL]|AC:[LMH]|Au:[MSN]|[CIA]:[NPC]|E:(U|POC|F|H|ND)|RL:(OF|TF|W|U|ND)|RC:(UC|UR|C|ND)|CDP:(N|L|LM|MH|H|ND)|TD:(N|L|M|H|ND)|[CIA]R:(L|M|H|ND))/)*(AV:[NAL]|AC:[LMH]|Au:[MSN]|[CIA]:[NPC]|E:(U|POC|F|H|ND)|RL:(OF|TF|W|U|ND)|RC:(UC|UR|C|ND)|CDP:(N|L|LM|MH|H|ND)|TD:(N|L|M|H|ND)|[CIA]R:(L|M|H|ND))$"
        ),
    ]
    accessVector: Optional[AccessVectorType] = None
    accessComplexity: Optional[AccessComplexityType] = None
    authentication: Optional[AuthenticationType] = None
    confidentialityImpact: Optional[CiaTypeModel] = None
    integrityImpact: Optional[CiaTypeModel] = None
    availabilityImpact: Optional[CiaTypeModel] = None
    baseScore: ScoreType
    exploitability: Optional[ExploitabilityType] = None
    remediationLevel: Optional[RemediationLevelType] = None
    reportConfidence: Optional[ReportConfidenceType] = None
    temporalScore: Optional[ScoreType] = None
    collateralDamagePotential: Optional[CollateralDamagePotentialType] = None
    targetDistribution: Optional[TargetDistributionType] = None
    confidentialityRequirement: Optional[CiaRequirementType] = None
    integrityRequirement: Optional[CiaRequirementType] = None
    availabilityRequirement: Optional[CiaRequirementType] = None
    environmentalScore: Optional[ScoreType] = None



"""
NPM Security Advisory to NVD CVE converter

This module implements basic functionality to query npm registry for security advisories
"""
import logging

import httpx
import orjson

from vdb.lib import config
from vdb.lib.nvd import NvdSource
from vdb.lib.utils import (
    compress_str,
    convert_md_references,
    convert_to_occurrence,
    fix_text,
    get_cvss3_from_vector,
    get_default_cve_data,
    serialize_vuln_list,
)

logging.basicConfig(
    level=logging.INFO, format="%(levelname)s [%(asctime)s] %(message)s"
)
LOG = logging.getLogger(__name__)


class NpmSource(NvdSource):
    """
    Npm source
    """

    def bulk_search(self, app_info, pkg_list):
        """
        Bulk search the resource instead of downloading the information

        :return: Vulnerability occurrences
        """
        payload = {**app_info}
        requires = {}
        dependencies = {}
        for pkg in pkg_list:
            vendor = None
            if isinstance(pkg, dict):
                vendor = pkg.get("vendor")
                name = pkg.get("name")
                version = pkg.get("version")
            else:
                tmp_a = pkg.split("|")
                version = tmp_a[len(tmp_a) - 1]
                name = tmp_a[len(tmp_a) - 2]
                if len(tmp_a) == 3:
                    vendor = tmp_a[0]
            key = name
            if vendor:
                key = f"{vendor}/{name}"
            requires[key] = version
            dependencies[key] = {"version": version}
        payload["requires"] = requires
        payload["dependencies"] = dependencies
        return convert_to_occurrence(serialize_vuln_list(self.fetch(payload)))

    def fetch(self, payload):
        client = httpx.Client(http2=True, follow_redirects=True, timeout=180)
        LOG.debug("Fetch npm advisory from %s", config.NPM_AUDIT_URL)
        r = client.post(url=config.NPM_AUDIT_URL, json=payload)
        json_data = r.json()
        return self.convert(json_data)

    def download_recent(self):
        """Method which downloads the recent CVE"""
        client = httpx.Client(http2=True, follow_redirects=True, timeout=180)
        url = config.NPM_ADVISORIES_URL + "?perPage=100&page=1"
        r = client.get(url=url)
        if r.ok:
            json_data = r.json()
            data = self.convert(json_data.get("objects"))
            if data:
                self.store(data)
            return data
        return []

    def download_all(self):
        """Download all historic cve data"""
        data_list = []
        client = httpx.Client(http2=True, follow_redirects=True, timeout=180)
        url = config.NPM_ADVISORIES_URL + "?perPage=100&page=1"
        for y in range(0, int(config.NPM_PAGES_COUNT)):
            r = client.get(url=url)
            if r.ok:
                json_data = r.json()
                data = self.convert(json_data.get("objects"))
                if data:
                    self.store(data)
                data_list += data
                if json_data.get("urls", {}).get("next"):
                    url = config.NPM_SERVER + json_data.get("urls").get("next")
                else:
                    break
        return data_list

    @staticmethod
    def get_version_ranges(version_str):
        """
        Version range formats used by npm
        <1.10.2
        <=4.0.13 || >=4.1.0 <4.1.2
        >=4.0.14 <4.1.0 || >=4.1.2
        :param version_str:
        :return: List of version ranges
        """
        version_list = []
        tmp_a = version_str.split("||")
        for ver in tmp_a:
            version_start_including = ""
            version_end_including = ""
            version_start_excluding = ""
            version_end_excluding = ""
            ver = ver.strip()
            ver = (
                ver.replace("<= ", "<=")
                .replace(">= ", ">=")
                .replace("< ", "<")
                .replace("> ", ">")
            )
            tmp_b = ver.split(" ")
            if tmp_b[0] == ">=" and len(tmp_b) > 1:
                version_start_including = tmp_b[1]
            elif tmp_b[0].startswith(">="):
                version_start_including = tmp_b[0].replace(">=", "")
            elif tmp_b[0].startswith(">"):
                version_start_excluding = tmp_b[0].replace(">", "")
            if tmp_b[-1].startswith("<="):
                version_end_including = tmp_b[-1].replace("<=", "")
            elif tmp_b[-1].startswith("<"):
                version_end_excluding = tmp_b[-1].replace("<", "")
            if (
                not version_start_including
                and not version_end_including
                and not version_start_excluding
                and not version_end_excluding
            ):
                version_start_including = version_str
            version_list.append(
                {
                    "version_start_including": version_start_including,
                    "version_end_including": version_end_including,
                    "version_start_excluding": version_start_excluding,
                    "version_end_excluding": version_end_excluding,
                }
            )
        return version_list

    def convert(self, adv_data):
        ret_data = []
        if isinstance(adv_data, list):
            for d in adv_data:
                self.to_vuln(d, ret_data)
        else:
            if adv_data.get("advisories"):
                for k, v in adv_data.get("advisories").items():
                    if v["deleted"]:
                        continue
                    self.to_vuln(v, ret_data)
        return ret_data

    def to_vuln(self, v, ret_data):
        assigner = "github_m"
        # Iterate the cve list if available
        cves = v.get("cves")
        if not cves:
            cves = ["{}-{}".format("NPM", v.get("id"))]
        for cve_id in cves:
            published_date = v["created"]
            last_modified_date = v["updated"]
            title = v.get("title", "")
            overview = v.get("overview", "")
            recommendation = v.get("recommendation", "")
            description = """# {}
{}
{}
            """.format(
                title, overview, recommendation
            )
            references = (
                [{"name": "npm advisory", "url": v.get("url")}] if v.get("url") else []
            )
            if v.get("references"):
                references = convert_md_references(v.get("references"))
            references = orjson.dumps(references, option=orjson.OPT_NAIVE_UTC)
            if isinstance(references, bytes):
                references = references.decode("utf-8", "ignore")
            severity = v.get("severity")
            vendor = "npm"
            product = v["module_name"]
            score, severity, vector_string, attack_complexity = get_default_cve_data(
                severity
            )
            cvss = v.get("cvss")
            user_interaction = "REQUIRED"
            exploitability_score = score
            metadata = v.get("metadata", {})
            if isinstance(metadata, dict) and metadata.get("exploitability"):
                exploitability_score = metadata.get("exploitability")
            if cvss:
                if cvss.get("score"):
                    score = cvss.get("score")
                if cvss.get("vectorString"):
                    vector_string = cvss.get("vectorString")
                    if vector_string:
                        cvss3_obj = get_cvss3_from_vector(vector_string)
                        if cvss3_obj:
                            exploitability_score = cvss3_obj.get("temporalScore")
                            attack_complexity = cvss3_obj.get("attackComplexity")
                            user_interaction = cvss3_obj.get("userInteraction")
            cwe_id = v.get("cwe")
            findings = v.get("findings")
            used_version = ""
            if findings and len(findings):
                used_version = findings[0].get("version")
            version = v["vulnerable_versions"]
            fix_version = v.get("patched_versions")
            version_ranges = self.get_version_ranges(version)
            fix_version_ranges = self.get_version_ranges(fix_version)
            vr = 0
            for ver in version_ranges:
                version_start_including = ver["version_start_including"]
                version_end_including = ver["version_end_including"]
                version_start_excluding = ver["version_start_excluding"]
                version_end_excluding = ver["version_end_excluding"]
                top_fix_version = (
                    fix_version_ranges[vr]
                    if len(fix_version_ranges) > vr
                    else fix_version_ranges[0]
                )
                fix_version_start_including = top_fix_version.get(
                    "version_start_including", ""
                )
                fix_version_end_including = top_fix_version.get(
                    "version_end_including", ""
                )
                fix_version_start_excluding = top_fix_version.get(
                    "version_start_excluding", ""
                )
                fix_version_end_excluding = top_fix_version.get(
                    "version_end_excluding", ""
                )
                description = fix_text(description)

                tdata = config.CVE_TPL % dict(
                    cve_id=cve_id,
                    cwe_id=cwe_id,
                    assigner=assigner,
                    references=references,
                    description="",
                    vectorString=vector_string,
                    vendor=vendor,
                    product=product,
                    version="*",
                    edition="*",
                    version_start_including=version_start_including,
                    version_end_including=version_end_including,
                    version_start_excluding=version_start_excluding,
                    version_end_excluding=version_end_excluding,
                    fix_version_start_including=fix_version_start_including,
                    fix_version_end_including=fix_version_end_including,
                    fix_version_start_excluding=fix_version_start_excluding,
                    fix_version_end_excluding=fix_version_end_excluding,
                    severity=severity,
                    attackComplexity=attack_complexity,
                    score=score,
                    userInteraction=user_interaction,
                    exploitabilityScore=exploitability_score,
                    publishedDate=published_date,
                    lastModifiedDate=last_modified_date,
                )
                try:
                    vuln = NvdSource.convert_vuln(orjson.loads(tdata))
                    vuln.description = compress_str(description)
                    ret_data.append([vuln, f"""{v["id"]}|{product}|{used_version}"""])
                except Exception as e:
                    LOG.debug(e)
                vr = vr + 1
        return ret_data
